"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  fromZodSchema: () => fromZodSchema,
  toZodSchema: () => toZodSchema,
  toZodString: () => toZodString
});
module.exports = __toCommonJS(src_exports);

// src/from-schema.ts
var import_core = require("@onetyped/core");
var import_zod = require("zod");
var fromZodSchema = (schema) => {
  const typeName = schema._def.typeName;
  switch (typeName) {
    case import_zod.ZodFirstPartyTypeKind.ZodString: {
      return (0, import_core.string)();
    }
    case import_zod.ZodFirstPartyTypeKind.ZodNumber: {
      return (0, import_core.number)();
    }
    case import_zod.ZodFirstPartyTypeKind.ZodBoolean: {
      return (0, import_core.boolean)();
    }
    case import_zod.ZodFirstPartyTypeKind.ZodNull: {
      return (0, import_core.nullType)();
    }
    case import_zod.ZodFirstPartyTypeKind.ZodUndefined: {
      return (0, import_core.undefinedType)();
    }
    case import_zod.ZodFirstPartyTypeKind.ZodUnknown: {
      return (0, import_core.unknown)();
    }
    case import_zod.ZodFirstPartyTypeKind.ZodLiteral: {
      return (0, import_core.literal)(schema._def.value);
    }
    case import_zod.ZodFirstPartyTypeKind.ZodVoid: {
      return (0, import_core.voidType)();
    }
    case import_zod.ZodFirstPartyTypeKind.ZodAny: {
      return (0, import_core.any)();
    }
    case import_zod.ZodFirstPartyTypeKind.ZodNever: {
      return (0, import_core.never)();
    }
    case import_zod.ZodFirstPartyTypeKind.ZodBigInt: {
      return (0, import_core.bigint)();
    }
    case import_zod.ZodFirstPartyTypeKind.ZodDate: {
      return (0, import_core.date)();
    }
    case import_zod.ZodFirstPartyTypeKind.ZodObject: {
      const propertySchemas = Object.fromEntries(
        Object.entries(schema._def.shape()).map(([key, value]) => [
          key,
          fromZodSchema(value)
        ])
      );
      return (0, import_core.object)(propertySchemas);
    }
    case import_zod.ZodFirstPartyTypeKind.ZodArray: {
      return (0, import_core.array)(fromZodSchema(schema._def.type));
    }
    case import_zod.ZodFirstPartyTypeKind.ZodSet: {
      return (0, import_core.set)(fromZodSchema(schema._def.type));
    }
    case import_zod.ZodFirstPartyTypeKind.ZodRecord: {
      return (0, import_core.record)(
        fromZodSchema(schema._def.keyType),
        fromZodSchema(schema._def.valueType)
      );
    }
    case import_zod.ZodFirstPartyTypeKind.ZodMap: {
      return (0, import_core.map)(
        fromZodSchema(schema._def.keyType),
        fromZodSchema(schema._def.valueType)
      );
    }
    case import_zod.ZodFirstPartyTypeKind.ZodFunction: {
      const argumentSchemas = schema._def.args.map(
        (argument) => fromZodSchema(argument)
      );
      const returnSchema = typeof schema._def.returns === "undefined" ? void 0 : fromZodSchema(schema._def.returns);
      return (0, import_core.func)({ arguments: argumentSchemas, return: returnSchema });
    }
    case import_zod.ZodFirstPartyTypeKind.ZodUnion: {
      const schemas = schema._def.options.map(
        (option) => fromZodSchema(option)
      );
      return (0, import_core.union)(schemas);
    }
    case import_zod.ZodFirstPartyTypeKind.ZodTuple: {
      const schemas = schema._def.items.map(
        (item) => fromZodSchema(item)
      );
      return (0, import_core.tuple)(schemas);
    }
    case import_zod.ZodFirstPartyTypeKind.ZodIntersection: {
      const schemas = schema._def.options.map(
        (option) => fromZodSchema(option)
      );
      return (0, import_core.intersection)(schemas);
    }
    case import_zod.ZodFirstPartyTypeKind.ZodOptional: {
      return (0, import_core.optional)(fromZodSchema(schema._def.innerType));
    }
    case import_zod.ZodFirstPartyTypeKind.ZodNullable: {
      return (0, import_core.nullable)(fromZodSchema(schema._def.innerType));
    }
    default: {
      throw new import_core.UnsupportedError("zod", typeName);
    }
  }
};

// src/to-schema.ts
var import_core2 = require("@onetyped/core");
var import_zod2 = require("zod");
var toZodSchema = (node, definitions = /* @__PURE__ */ new Map()) => {
  const zodSchemaDefinitions = /* @__PURE__ */ new Map();
  const schema = toZodSchemaInternal(node, definitions, zodSchemaDefinitions);
  return schema;
};
var toZodSchemaInternal = (node, nodeDefinitions, zodSchemaDefinitions) => {
  switch (node.typeName) {
    case "definitionReference": {
      const schemaBag = zodSchemaDefinitions.get(node.identifier);
      if (schemaBag) {
        return import_zod2.z.lazy(() => {
          return schemaBag.schema;
        });
      }
      const nodeDefinition = nodeDefinitions.get(node.identifier);
      const resolvedSchemaBag = { schema: import_zod2.z.unknown() };
      zodSchemaDefinitions.set(node.identifier, resolvedSchemaBag);
      resolvedSchemaBag.schema = toZodSchemaInternal(
        nodeDefinition,
        nodeDefinitions,
        zodSchemaDefinitions
      );
      return import_zod2.z.lazy(() => resolvedSchemaBag.schema);
    }
    case "string": {
      return import_zod2.z.string();
    }
    case "number": {
      return import_zod2.z.number();
    }
    case "boolean": {
      return import_zod2.z.boolean();
    }
    case "null": {
      return import_zod2.z.null();
    }
    case "undefined": {
      return import_zod2.z.undefined();
    }
    case "literal": {
      return import_zod2.z.literal(node.type);
    }
    case "unknown": {
      return import_zod2.z.unknown();
    }
    case "void": {
      return import_zod2.z.void();
    }
    case "any": {
      return import_zod2.z.any();
    }
    case "never": {
      return import_zod2.z.never();
    }
    case "regexp": {
      return import_zod2.z.instanceof(RegExp);
    }
    case "bigint": {
      return import_zod2.z.bigint();
    }
    case "date": {
      return import_zod2.z.date();
    }
    case "file": {
      return import_zod2.z.instanceof(File);
    }
    case "blob": {
      return import_zod2.z.instanceof(Blob);
    }
    case "formData": {
      return import_zod2.z.instanceof(FormData);
    }
    case "object": {
      const propertySchemas = Object.fromEntries(
        (0, import_core2.mapObjectNode)(
          node,
          (node2) => toZodSchemaInternal(node2, nodeDefinitions, zodSchemaDefinitions)
        )
      );
      return import_zod2.z.object(propertySchemas);
    }
    case "array": {
      return import_zod2.z.array(
        toZodSchemaInternal(node.type, nodeDefinitions, zodSchemaDefinitions)
      );
    }
    case "set": {
      return import_zod2.z.set(
        toZodSchemaInternal(node.type, nodeDefinitions, zodSchemaDefinitions)
      );
    }
    case "record": {
      return import_zod2.z.record(
        toZodSchemaInternal(node.key, nodeDefinitions, zodSchemaDefinitions),
        toZodSchemaInternal(node.value, nodeDefinitions, zodSchemaDefinitions)
      );
    }
    case "map": {
      return import_zod2.z.map(
        toZodSchemaInternal(node.key, nodeDefinitions, zodSchemaDefinitions),
        toZodSchemaInternal(node.value, nodeDefinitions, zodSchemaDefinitions)
      );
    }
    case "function": {
      const argumentSchemas = import_zod2.z.tuple(
        node.arguments.map(
          (argument) => toZodSchemaInternal(argument, nodeDefinitions, zodSchemaDefinitions)
        )
      );
      if (node.return) {
        return import_zod2.z.function(
          argumentSchemas,
          toZodSchemaInternal(
            node.return,
            nodeDefinitions,
            zodSchemaDefinitions
          )
        );
      }
      return import_zod2.z.function(argumentSchemas);
    }
    case "union": {
      if (node.types.length === 1)
        return toZodSchemaInternal(
          node.types[0],
          nodeDefinitions,
          zodSchemaDefinitions
        );
      const schemas = (0, import_core2.mapMultipleInnerNode)(
        node,
        (node2) => toZodSchemaInternal(node2, nodeDefinitions, zodSchemaDefinitions)
      );
      return import_zod2.z.union(schemas);
    }
    case "tuple": {
      const schemas = (0, import_core2.mapMultipleInnerNode)(
        node,
        (node2) => toZodSchemaInternal(node2, nodeDefinitions, zodSchemaDefinitions)
      );
      return import_zod2.z.tuple(schemas);
    }
    case "intersection": {
      const types = [...node.types];
      let schema = toZodSchemaInternal(
        types.pop(),
        nodeDefinitions,
        zodSchemaDefinitions
      );
      for (const type of types) {
        schema = schema.and(
          toZodSchemaInternal(type, nodeDefinitions, zodSchemaDefinitions)
        );
      }
      return schema;
    }
    case "named": {
      return toZodSchemaInternal(
        node.node,
        nodeDefinitions,
        zodSchemaDefinitions
      );
    }
    case "iterator": {
      return import_zod2.z.unknown();
    }
    case "symbol": {
      return import_zod2.z.unknown();
    }
    case "custom": {
      return import_zod2.z.unknown();
    }
    case "circular": {
      return import_zod2.z.unknown();
    }
  }
};

// src/to-string.ts
var import_core3 = require("@onetyped/core");
var createZodMethod = (method, argument_) => {
  return `z.${method}(${argument_ ?? ""})`;
};
var toZodString = (node) => {
  switch (node.typeName) {
    case "definitionReference": {
      return createZodMethod("lazy", `() => ${node.identifier}`);
    }
    case "string": {
      return createZodMethod("string");
    }
    case "number": {
      return createZodMethod("number");
    }
    case "boolean": {
      return createZodMethod("boolean");
    }
    case "null": {
      return createZodMethod("null");
    }
    case "undefined": {
      return createZodMethod("undefined");
    }
    case "literal": {
      return createZodMethod("literal", (0, import_core3.serializePrimitive)(node.type));
    }
    case "unknown": {
      return createZodMethod("unknown");
    }
    case "void": {
      return createZodMethod("void");
    }
    case "any": {
      return createZodMethod("any");
    }
    case "never": {
      return createZodMethod("never");
    }
    case "regexp": {
      return createZodMethod("instanceof", "RegExp");
    }
    case "bigint": {
      return createZodMethod("bigint");
    }
    case "date": {
      return createZodMethod("date");
    }
    case "file": {
      return createZodMethod("instanceof", "File");
    }
    case "blob": {
      return createZodMethod("instanceof", "Blob");
    }
    case "formData": {
      return createZodMethod("instanceof", "FormData");
    }
    case "iterator": {
      return createZodMethod("unknown");
    }
    case "symbol": {
      return createZodMethod("unknown");
    }
    case "custom": {
      return createZodMethod("unknown");
    }
    case "object": {
      const propertySchemas = (0, import_core3.mapObjectNode)(node, toZodString).map(([key, zodType]) => `${JSON.stringify(key)}: ${zodType}`).join(", ");
      return createZodMethod("object", `{ ${propertySchemas} }`);
    }
    case "array": {
      return createZodMethod("array", toZodString(node.type));
    }
    case "set": {
      return createZodMethod("set", toZodString(node.type));
    }
    case "record": {
      return createZodMethod(
        "record",
        `${toZodString(node.key)}, ${toZodString(node.value)}`
      );
    }
    case "map": {
      return createZodMethod(
        "map",
        `${toZodString(node.key)}, ${toZodString(node.value)}`
      );
    }
    case "function": {
      if (node.arguments.length === 0 && typeof node.return === "undefined")
        return createZodMethod("function");
      const argumentSchemas = node.arguments.map(
        (argument) => toZodString(argument)
      );
      const tupleArguments = createZodMethod(
        "tuple",
        `[${argumentSchemas.join(", ")}]`
      );
      if (typeof node.return === "undefined")
        return createZodMethod("function", tupleArguments);
      const returnSchema = toZodString(node.return);
      return createZodMethod("function", `${tupleArguments}, ${returnSchema}`);
    }
    case "union": {
      if (node.types.length === 1)
        return toZodString(node.types[0]);
      const schemas = (0, import_core3.mapMultipleInnerNode)(node, toZodString).join(", ");
      return createZodMethod("union", `[${schemas}]`);
    }
    case "tuple": {
      const schemas = (0, import_core3.mapMultipleInnerNode)(node, toZodString).join(", ");
      return createZodMethod("tuple", `[${schemas}]`);
    }
    case "intersection": {
      const types = [...node.types];
      let schema = toZodString(types.pop());
      for (const type of types) {
        schema = `${schema}.and(${toZodString(type)})`;
      }
      return schema;
    }
    case "named": {
      throw new import_core3.UnsupportedError("zod", "named");
    }
    case "circular": {
      throw new import_core3.UnsupportedError("zod", "circular");
    }
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  fromZodSchema,
  toZodSchema,
  toZodString
});
